@@ -18,6 +18,7 @@
 static int rflag;
 static int Rflag;
 static int qflag;
+static int Fflag;
 static int Hflag;
 static int Lflag;
 static int Nflag;
@@ -28,11 +29,15 @@ static char *hflag = defaulthflags;
 static char *xflag;
 static char *Oflag;
 
+static char fallback_ct[] = "text/plain";
+
 struct message *filters;
 
 static int mimecount;
 static int safe_output;
 
+static int reply_found;
+
 static char defaultAflags[] = "text/plain:text/html";
 static char *Aflag = defaultAflags;
 
@@ -42,7 +47,7 @@ printable(int c)
 	return (unsigned)c-0x20 < 0x5f;
 }
 
-int
+size_t
 print_ascii(char *body, size_t bodylen)
 {
 	if (safe_output) {
@@ -94,7 +99,7 @@ print_u8recode(char *body, size_t bodylen, char *srcenc)
 			final_char = bufptr[-1];
 		}
 
-		if (r != (size_t)-1) {	// done, flush iconv
+		if (r != (size_t)-1) {  // done, flush iconv
 			bufptr = buf;
 			buflen = sizeof buf;
 			r = iconv(ic, 0, 0, &bufptr, &buflen);
@@ -181,7 +186,7 @@ render_mime(int depth, struct message *msg, char *body, size_t bodylen)
 {
 	char *ct = blaze822_hdr(msg, "content-type");
 	if (!ct)
-		ct = "text/x-unknown";
+		ct = fallback_ct;
 	char *mt = mimetype(ct);
 	char *tlmt = tlmimetype(ct);
 	char *filename = mime_filename(msg);
@@ -215,11 +220,14 @@ render_mime(int depth, struct message *msg, char *body, size_t bodylen)
 		size_t outlen;
 		int e = filter(body, bodylen, cmd, &output, &outlen);
 
-		if (e == 0) { // replace output
+		if (e == 0 || e == 62) { // replace output (62 == raw)
 			if (!Nflag)
 				printf(" render=\"%s\" ---\n", cmd);
 			if (outlen) {
-				print_ascii(output, outlen);
+				if (e == 0)
+					print_ascii(output, outlen);
+				else
+					fwrite(output, 1, outlen, stdout);
 				if (output[outlen-1] != '\n')
 					putchar('\n');
 			}
@@ -256,7 +264,8 @@ nofilter:
 		if (!Nflag)
 			printf(" ---\n");
 
-		if (strncmp(ct, "text/", 5) == 0) {
+		if (strncmp(ct, "text/", 5) == 0 ||
+		    strncmp(ct, "message/delivery-status", 23) == 0) {
 			char *charset = 0, *cs, *cse;
 			if (blaze822_mime_parameter(ct, "charset", &cs, &cse))
 				charset = strndup(cs, cse-cs);
@@ -265,7 +274,7 @@ nofilter:
 			    strcasecmp(charset, "utf8") == 0 ||
 			    strcasecmp(charset, "us-ascii") == 0) {
 				print_ascii(body, bodylen);
-				if (body[bodylen-1] != '\n')
+				if (bodylen > 0 && body[bodylen-1] != '\n')
 					putchar('\n');
 			} else {
 				print_u8recode(body, bodylen, charset);
@@ -309,7 +318,7 @@ choose_alternative(struct message *msg, int depth)
 		m++;
 		char *ict = blaze822_hdr(imsg, "content-type");
 		if (!ict)
-			ict = "text/x-unknown";
+			ict = fallback_ct;
 		char *imt = mimetype(ict);
 
 		char *s = strstr(Aflag, imt);
@@ -333,7 +342,7 @@ choose_alternative(struct message *msg, int depth)
 blaze822_mime_action
 reply_mime(int depth, struct message *msg, char *body, size_t bodylen)
 {
-	(void) depth;
+	(void)depth;
 
 	char *ct = blaze822_hdr(msg, "content-type");
 	char *mt = mimetype(ct);
@@ -350,6 +359,7 @@ reply_mime(int depth, struct message *msg, char *body, size_t bodylen)
 			print_ascii(body, bodylen);
 		else
 			print_u8recode(body, bodylen, charset);
+		reply_found++;
 		free(charset);
 	}
 
@@ -362,11 +372,11 @@ reply_mime(int depth, struct message *msg, char *body, size_t bodylen)
 blaze822_mime_action
 list_mime(int depth, struct message *msg, char *body, size_t bodylen)
 {
-	(void) body;
+	(void)body;
 
 	char *ct = blaze822_hdr(msg, "content-type");
 	if (!ct)
-		ct = "text/x-unknown";
+		ct = fallback_ct;
 	char *mt = mimetype(ct);
 	char *filename = mime_filename(msg);
 
@@ -374,12 +384,15 @@ list_mime(int depth, struct message *msg, char *body, size_t bodylen)
 	print_filename(filename);
 	printf("\n");
 
+	free(mt);
 	return MIME_CONTINUE;
 }
 
 void
 list(char *file)
 {
+	while (*file == ' ' || *file == '\t')
+		file++;
 	struct message *msg = blaze822_file(file);
 	if (!msg)
 		return;
@@ -391,10 +404,15 @@ list(char *file)
 void
 reply(char *file)
 {
+	while (*file == ' ' || *file == '\t')
+		file++;
 	struct message *msg = blaze822_file(file);
 	if (!msg)
 		return;
+	reply_found = 0;
 	blaze822_walk_mime(msg, 0, reply_mime);
+	if (!reply_found)
+		exit(1);
 }
 
 static int extract_argc;
@@ -438,7 +456,7 @@ writefile(char *name, char *buf, ssize_t len)
 blaze822_mime_action
 extract_mime(int depth, struct message *msg, char *body, size_t bodylen)
 {
-	(void) depth;
+	(void)depth;
 
 	char *filename = mime_filename(msg);
 
@@ -494,7 +512,7 @@ extract_mime(int depth, struct message *msg, char *body, size_t bodylen)
 					writefile(bufptr, body, bodylen);
 				}
 			} else if (filename &&
-				   fnmatch(a, filename, FNM_PATHNAME) == 0) {
+			    fnmatch(a, filename, FNM_PATHNAME) == 0) {
 				// extract by name
 				if (extract_stdout) {
 					if (rflag) {
@@ -553,7 +571,7 @@ print_date_header(char *v)
 
 	printf("Date: ");
 	print_ascii(v, strlen(v));
-	
+
 	time_t t = blaze822_date(v);
 	if (t == -1) {
 		printf(" (invalid)");
@@ -561,7 +579,7 @@ print_date_header(char *v)
 		printf(" (");
 		time_t d = t < now ? now - t : t - now;
 
-		int l;
+		char l;
 		if (d > 60*60*24*7*52) l = 'y';
 		else if (d > 60*60*24*7) l = 'w';
 		else if (d > 60*60*24) l = 'd';
@@ -569,57 +587,57 @@ print_date_header(char *v)
 		else if (d > 60) l = 'm';
 		else l = 's';
 		int p = 3;
-		
-		int z;
-		switch(l) {
+
+		long z;
+		switch (l) {
 		case 'y':
 			z = d / (60*60*24*7*52);
 			d = d % (60*60*24*7*52);
 			if (z > 0) {
-				printf("%d year%s", z, z>1 ? "s" : "");
+				printf("%ld year%s", z, z > 1 ? "s" : "");
 				if (!--p) break;
 				printf(", ");
 			}
-			/* FALL THROUGH */
+		/* FALL THROUGH */
 		case 'w':
 			z = d / (60*60*24*7);
 			d = d % (60*60*24*7);
 			if (z > 0) {
-				printf("%d week%s", z, z>1 ? "s" : "");
+				printf("%ld week%s", z, z > 1 ? "s" : "");
 				if (!--p) break;
 				printf(", ");
 			}
-			/* FALL THROUGH */
+		/* FALL THROUGH */
 		case 'd':
 			z = d / (60*60*24);
 			d = d % (60*60*24);
 			if (z > 0) {
-				printf("%d day%s", z, z>1 ? "s" : "");
+				printf("%ld day%s", z, z > 1 ? "s" : "");
 				if (!--p) break;
 				printf(", ");
 			}
-			/* FALL THROUGH */
+		/* FALL THROUGH */
 		case 'h':
 			z = d / (60*60);
 			d = d % (60*60);
 			if (z > 0) {
-				printf("%d hour%s", z, z>1 ? "s" : "");
+				printf("%ld hour%s", z, z > 1 ? "s" : "");
 				if (!--p) break;
 				printf(", ");
 			}
-			/* FALL THROUGH */
+		/* FALL THROUGH */
 		case 'm':
 			z = d / (60);
 			d = d % (60);
 			if (z > 0) {
-				printf("%d minute%s", z, z>1 ? "s" : "");
+				printf("%ld minute%s", z, z > 1 ? "s" : "");
 				if (!--p) break;
 				printf(", ");
 			}
-			/* FALL THROUGH */
+		/* FALL THROUGH */
 		case 's':
 			z = d;
-			printf("%d second%s", z, z>1 ? "s" : "");
+			printf("%ld second%s", z, z > 1 ? "s" : "");
 		}
 
 		if (t < now)
@@ -704,7 +722,7 @@ show(char *file)
 
 	printf("\n");
 
-	if (rflag || !blaze822_check_mime(msg)) {  // raw body
+	if (rflag) {  // raw body
 		print_ascii(blaze822_body(msg), blaze822_bodylen(msg));
 		goto done;
 	}
@@ -722,12 +740,13 @@ main(int argc, char *argv[])
 	pid_t pid1 = -1, pid2 = -1;
 
 	int c;
-	while ((c = getopt(argc, argv, "h:A:qrtHLNx:O:Rn")) != -1)
-		switch(c) {
+	while ((c = getopt(argc, argv, "h:A:qrtFHLNx:O:Rn")) != -1)
+		switch (c) {
 		case 'h': hflag = optarg; break;
 		case 'A': Aflag = optarg; break;
 		case 'q': qflag = 1; break;
 		case 'r': rflag = 1; break;
+		case 'F': Fflag = 1; break;
 		case 'H': Hflag = 1; break;
 		case 'L': Lflag = 1; break;
 		case 'N': Nflag = 1; break;
@@ -738,7 +757,7 @@ main(int argc, char *argv[])
 		case 'n': nflag = 1; break;
 		default:
 			fprintf(stderr,
-			    "Usage: mshow [-h headers] [-A mimetypes] [-nqrHLN] [msgs...]\n"
+			    "Usage: mshow [-h headers] [-A mimetypes] [-nqrFHLN] [msgs...]\n"
 			    "	    mshow -x msg parts...\n"
 			    "	    mshow -O msg parts...\n"
 			    "	    mshow -t msgs...\n"
@@ -753,8 +772,13 @@ main(int argc, char *argv[])
 	if (safe_output && isatty(1)) {
 		char *pg;
 		pg = getenv("MBLAZE_PAGER");
-		if (!pg)
+		if (!pg) {
 			pg = getenv("PAGER");
+			if (pg && strcmp(pg, "less") == 0) {
+				static char lesscmd[] = "less -RFXe";
+				pg = lesscmd;
+			}
+		}
 		if (pg && *pg && strcmp(pg, "cat") != 0) {
 			pid2 = pipeto(pg);
 			if (pid2 < 0)
@@ -778,7 +802,7 @@ main(int argc, char *argv[])
 	} else if (Rflag) { // render for reply
 		blaze822_loop(argc-optind, argv+optind, reply);
 	} else { // show
-		if (!(qflag || rflag)) {
+		if (!(qflag || rflag || Fflag)) {
 			char *f = getenv("MAILFILTER");
 			if (!f)
 				f = blaze822_home_file("filter");
